from aiogram import Router, F
from aiogram.types import Message
from aiogram.fsm.context import FSMContext
from aiogram.filters import BaseFilter
from tortoise.models import Model
from typing import Union, List, Type, Optional

from filters.filters import ChatTypeFilter


class HelpCenterRouter:
    """
    Класс HelpCenterRouter предназначен для настройки и управления роутами в aiogram,
    которые обрабатывают различные типы сообщений и перенаправляют их в чат поддержки.

    Параметры:
    - user_model (Type[Model]): Модель пользователя, используемая для поиска и сохранения данных.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - thread_field (str): Поле для идентификатора темы, по умолчанию 'thread_id'.
    - settings_model (Optional[Type[Model]]): Модель настроек, используемая для получения ID чата поддержки.
    - help_chat_field (str): Поле для идентификатора чата поддержки, по умолчанию 'help_chat_id'.
    - help_chat_id (Optional[str]): Идентификатор чата поддержки, если не используется модель настроек.
    - enable_text (bool): Включить обработку текстовых сообщений, по умолчанию True.
    - enable_image (bool): Включить обработку изображений, по умолчанию True.
    - enable_location (bool): Включить обработку местоположений, по умолчанию True.
    - enable_document (bool): Включить обработку документов, по умолчанию True.
    - enable_video (bool): Включить обработку видео, по умолчанию True.
    - enable_video_note (bool): Включить обработку видеозаметок, по умолчанию True.
    - enable_voice (bool): Включить обработку голосовых сообщений, по умолчанию True.
    - enable_contact (bool): Включить обработку контактов, по умолчанию True.

    Методы:
    - get_router() -> Router: Возвращает настроенный роутер для использования в диспетчере aiogram.

    Примечание:
    - Если не указан help_chat_id и не предоставлена settings_model, будет вызвано исключение ValueError.
    - Обработчики сообщений автоматически перенаправляют сообщения из личных чатов в чат поддержки
      и обратно, если они не были обработаны ни одним из предыдущих обработчиков.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id',
                 thread_field: str = 'thread_id',
                 settings_model: Optional[Type[Model]] = None,
                 help_chat_field: str = 'help_chat_id',
                 help_chat_id: Optional[str] = None,
                 enable_text: bool = True,
                 enable_image: bool = True,
                 enable_location: bool = True,
                 enable_document: bool = True,
                 enable_video: bool = True,
                 enable_video_note: bool = True,
                 enable_voice: bool = True,
                 enable_contact: bool = True):
        """
        :param help_chat_id: ID чата поддержки
        :param user_model: Django/TortoiseORM модель пользователя
        :param user_fields: Поля, по которым искать пользователя (например, ['user_id', 'thread_id'])
        """
        self.user_model = user_model
        self.id_field = id_field
        self.thread_field = thread_field
        self.settings_model = settings_model
        self.help_chat_field = help_chat_field
        self.help_chat_id = help_chat_id

        self.enable_text = enable_text
        self.enable_image = enable_image
        self.enable_location = enable_location
        self.enable_document = enable_document
        self.enable_video = enable_video
        self.enable_video_note = enable_video_note
        self.enable_voice = enable_voice
        self.enable_contact = enable_contact

        self.router = Router()

        if not self.help_chat_id and not self.settings_model:
            raise ValueError("help_chat_id или settings_model должны быть предоставлены")
        
        self._setup_handlers()

    async def __get_user(self, identifier):
        user = await self.user_model.get_or_none(**{self.id_field: identifier})
        return user

    async def __get_user_by_thread_id(self, identifier):
        user = await self.user_model.get_or_none(**{self.thread_field: identifier})
        return user
    
    async def __get_help_chat(self):
        if self.settings_model:
            settings = await self.settings_model.first()
            return getattr(settings, self.help_chat_field, None)
        return self.help_chat_id
    
    def _setup_handlers(self):
        if self.enable_text:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.text)
            async def text_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_message(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            text=message.text,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.text)
            async def text_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_message(
                                chat_id=getattr(user, self.id_field, None),
                                text=message.text,
                            )
                        except:
                            pass

        if self.enable_image:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.photo)
            async def image_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_photo(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            caption=message.caption,
                            photo=message.photo[-1].file_id,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.photo)
            async def image_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_photo(
                                chat_id=getattr(user, self.id_field, None),
                                caption=message.caption,
                                photo=message.photo[-1].file_id,
                            )
                        except:
                            pass

        if self.enable_document:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.document)
            async def document_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_document(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            document=message.document.file_id,
                            caption=message.caption,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.document)
            async def document_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_document(
                                chat_id=getattr(user, self.id_field, None),
                                document=message.document.file_id,
                                caption=message.caption,
                            )
                        except:
                            pass

        if self.enable_video:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.video)
            async def video_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_video(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            video=message.video.file_id,
                            caption=message.caption,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.video)
            async def video_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_video(
                                chat_id=getattr(user, self.id_field, None),
                                video=message.video.file_id,
                                caption=message.caption,
                            )
                        except:
                            pass

        if self.enable_voice:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.voice)
            async def voice_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_voice(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            voice=message.voice.file_id,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.voice)
            async def voice_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_voice(
                                chat_id=getattr(user, self.id_field, None),
                                voice=message.voice.file_id,
                            )
                        except:
                            pass

        if self.enable_contact:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.contact)
            async def contact_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_contact(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            phone_number=message.contact.phone_number,
                            first_name=message.contact.first_name,
                            last_name=message.contact.last_name,
                            vcard=message.contact.vcard,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.contact)
            async def contact_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_contact(
                                chat_id=getattr(user, self.id_field, None),
                                phone_number=message.contact.phone_number,
                                first_name=message.contact.first_name,
                                last_name=message.contact.last_name,
                                vcard=message.contact.vcard,
                            )
                        except:
                            pass

        if self.enable_video_note:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.video_note)
            async def video_note_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_video_note(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            video_note=message.video_note.file_id,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.video_note)
            async def video_note_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_video_note(
                                chat_id=getattr(user, self.id_field, None),
                                video_note=message.video_note.file_id,
                            )
                        except:
                            pass

        if self.enable_location:
            @self.router.message(ChatTypeFilter(chat_type='private'), F.location)
            async def location_to_help_center(message: Message, state: FSMContext):
                user = await self.__get_user(message.from_user.id)
                help_chat_id = await self.__get_help_chat()
                if user:
                    try:
                        await message.bot.send_location(
                            chat_id=help_chat_id,
                            message_thread_id=getattr(user, self.thread_field, None),
                            latitude=message.location.latitude,
                            longitude=message.location.longitude,
                        )
                    except:
                        pass

            @self.router.message(ChatTypeFilter(chat_type=['group', 'supergroup']), F.location)
            async def location_from_help_center(message: Message, state: FSMContext):
                if message.message_thread_id:
                    user = await self.__get_user_by_thread_id(message.message_thread_id)
                    help_chat_id = await self.__get_help_chat()
                    if user and str(help_chat_id) == str(message.chat.id):
                        try:
                            await message.bot.send_location(
                                chat_id=getattr(user, self.id_field, None),
                                latitude=message.location.latitude,
                                longitude=message.location.longitude,
                            )
                        except:
                            pass

    def get_router(self) -> Router:
        return self.router
