import datetime
import pytz

from typing import Any, Awaitable, Callable, Dict, Type, Optional
from aiogram import BaseMiddleware
from aiogram.types import TelegramObject, CallbackQuery
from tortoise.models import Model


class UpdateLastActivityMiddleware(BaseMiddleware):
    """
    Middleware для обновления   .

    Параметры:
    - user_model (Type[Model]): Модель пользователя, обязательный параметр.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - last_activity_field (str): Поле для последней активности пользователя, по умолчанию 'last_activity'.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id', 
                 last_activity_field: str = 'last_activity', 
                 ):
        self.user_model = user_model
        self.id_field = id_field
        self.last_activity_field = last_activity_field

    async def __call__(
        self,
        handler: Callable[[TelegramObject, Dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: Dict[str, Any],
        *args,
        **kwargs
    ):
        user_id = data["event_from_user"].id

        user, _ = await self.user_model.get_or_create(**{self.id_field: user_id})

        setattr(user, self.last_activity_field, datetime.datetime.now(pytz.utc))
        await user.save()

        return await handler(event, data)
    

class UpdateUserDataMiddleware(BaseMiddleware):
    """
    Middleware для обновления данных пользователя.

    Параметры:
    - user_model (Type[Model]): Модель пользователя, обязательный параметр.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - username_field (str): Поле для имени пользователя, по умолчанию 'username'.
    - name_field (str): Поле для полного имени пользователя, по умолчанию 'name'.
    - update_name (bool): Флаг для обновления имени, по умолчанию True.
    - update_username (bool): Флаг для обновления имени пользователя, по умолчанию True.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id', 
                 username_field: str = 'username', 
                 name_field: str = 'name',
                 update_name: bool = True,
                 update_username: bool = True):
        self.user_model = user_model
        self.id_field = id_field
        self.username_field = username_field
        self.name_field = name_field
        self.update_name = update_name
        self.update_username = update_username

    async def __call__(
        self,
        handler: Callable[[TelegramObject, Dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: Dict[str, Any],
        *args,
        **kwargs
    ):
        user_id = data["event_from_user"].id
        username = data["event_from_user"].username
        name = data["event_from_user"].full_name

        user, _ = await self.user_model.get_or_create(**{self.id_field: user_id})

        if self.update_username:
            setattr(user, self.username_field, username)
        if self.update_name:
            setattr(user, self.name_field, name)
        await user.save()

        return await handler(event, data)


class CreateThreadMiddleware(BaseMiddleware):
    """
    Middleware для создания топика в чате.

    Параметры:
    - user_model (Type[Model]): Модель пользователя, обязательный параметр.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - username_field (str): Поле для имени пользователя, по умолчанию 'username'.
    - thread_field (str): Поле для идентификатора темы, по умолчанию 'thread_id'.
    - settings_model (Optional[Type[Model]]): Модель настроек, необязательный параметр.
    - help_chat_field (str): Поле для идентификатора чата помощи, по умолчанию 'help_chat_id'.
    - help_chat_id (Optional[str]): Идентификатор чата помощи, необязательный параметр.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id', 
                 username_field: str = 'username',
                 thread_field: str = 'thread_id',
                 settings_model: Optional[Type[Model]] = None,
                 help_chat_field: str = 'help_chat_id',
                 help_chat_id: Optional[str] = None):
        self.user_model = user_model
        self.id_field = id_field
        self.username_field = username_field
        self.thread_field = thread_field
        self.settings_model = settings_model
        self.help_chat_field = help_chat_field
        self.help_chat_id = help_chat_id

    async def __call__(
        self,
        handler: Callable[[TelegramObject, Dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: Dict[str, Any],
        *args,
        **kwargs
    ):
        user_id = data["event_from_user"].id
        user, _ = await self.user_model.get_or_create(**{self.id_field: user_id})
        thread_id = getattr(user, self.thread_field)
        if not thread_id:
            username = getattr(user, self.username_field)
            if username:
                result = f'@{username} {user_id}'
            else:
                result = str(user_id)
            
            if not self.help_chat_id:
                if not self.settings_model:
                    raise ValueError("help_chat_id или settings_model должны быть предоставлены")
                base_settings = await self.settings_model.first()
                self.help_chat_id = getattr(base_settings, self.help_chat_field)
            
            new_thread = await event.bot.create_forum_topic(
                chat_id=self.help_chat_id,
                name=result,
            )
            setattr(user, self.thread_field, new_thread.message_thread_id)
            await user.save(update_fields=[self.thread_field])

        return await handler(event, data)
    

class BlackListMiddleware(BaseMiddleware):
    """
    Middleware для проверки черного списка.

    Параметры:
    - user_model (Type[Model]): Модель пользователя, обязательный параметр.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - blacklist_field (str): Поле для статуса блокировки, по умолчанию 'is_blocked'.
    - text_model (Optional[Type[Model]]): Модель текста, необязательный параметр.
    - slug (Optional[str]): Слаг для поиска текста, необязательный параметр.
    - text_field (str): Поле для текста, по умолчанию 'text'.
    - in_black_list_text (Optional[str]): Текст для ответа, если пользователь в черном списке, необязательный параметр.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id', 
                 blacklist_field: str = 'is_blocked',
                 text_model: Optional[Type[Model]] = None,
                 slug: Optional[str] = None,
                 text_field: str = 'text',
                 in_black_list_text: Optional[str] = None):
        self.user_model = user_model
        self.id_field = id_field
        self.blacklist_field = blacklist_field
        self.text_model = text_model
        self.slug = slug
        self.text_field = text_field
        self.in_black_list_text = in_black_list_text
    
    async def __call__(
        self,
        handler: Callable[[TelegramObject, Dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: Dict[str, Any],
        *args,
        **kwargs
    ):
        user_id = data["event_from_user"].id
        user, _ = await self.user_model.get_or_create(**{self.id_field: user_id})
        
        if getattr(user, self.blacklist_field):
            try:
                if not self.in_black_list_text:
                    if not self.text_model or not self.slug or not self.text_field:
                        raise ValueError("text_model или in_black_list_text должны быть предоставлены")
                    reply_text = await self.text_model.get(slug=self.slug)
                    reply_text = getattr(reply_text, self.text_field)
                else:
                    reply_text = self.in_black_list_text

                if isinstance(event, CallbackQuery):
                    await event.answer(
                        text=reply_text,
                        show_alert=True
                    )
                else:
                    await event.answer(
                        text=reply_text,
                    )
            except:
                pass

            return
        
        return await handler(event, data)


class GlobalMiddleware(BaseMiddleware):
    """
    Глобальный middleware, включает функционал:
    - проверки черного списка
    - обновления данных пользователя
    - создания топика в чате
    - обновления последней активности пользователя

    Параметры:
    - user_model (Type[Model]): Модель пользователя, обязательный параметр.
    - id_field (str): Поле для идентификатора пользователя, по умолчанию 'user_id'.
    - username_field (str): Поле для имени пользователя, по умолчанию 'username'.
    - name_field (str): Поле для полного имени пользователя, по умолчанию 'name'.
    - thread_field (str): Поле для идентификатора темы, по умолчанию 'thread_id'.
    - blacklist_field (str): Поле для статуса блокировки, по умолчанию 'is_blocked'.
    - last_activity_field (str): Поле для последней активности пользователя, по умолчанию 'last_activity'.
    - settings_model (Optional[Type[Model]]): Модель настроек, необязательный параметр.
    - help_chat_field (str): Поле для идентификатора чата помощи, по умолчанию 'help_chat_id'.
    - help_chat_id (Optional[str]): Идентификатор чата помощи, необязательный параметр.
    - text_model (Optional[Type[Model]]): Модель текста, необязательный параметр.
    - slug (Optional[str]): Слаг для поиска текста, необязательный параметр.
    - text_field (str): Поле для текста, по умолчанию 'text'.
    - in_black_list_text (Optional[str]): Текст для ответа, если пользователь в черном списке, необязательный параметр.
    - update_name (bool): Флаг для обновления имени, по умолчанию True.
    - update_username (bool): Флаг для обновления имени пользователя, по умолчанию True.
    - enable_update_user_data (bool): Включить обновление данных пользователя, по умолчанию True.
    - enable_create_thread (bool): Включить создание темы, по умолчанию True.
    - enable_blacklist (bool): Включить проверку черного списка, по умолчанию True.
    - enable_last_activity (bool): Включить обновление последней активности, по умолчанию True.
    """
    def __init__(self, 
                 user_model: Type[Model], 
                 id_field: str = 'user_id', 
                 username_field: str = 'username', 
                 name_field: str = 'name',
                 thread_field: str = 'thread_id',
                 blacklist_field: str = 'is_blocked',
                 last_activity_field: str = 'last_activity',
                 settings_model: Optional[Type[Model]] = None,
                 help_chat_field: str = 'help_chat_id',
                 help_chat_id: Optional[str] = None,
                 text_model: Optional[Type[Model]] = None,
                 slug: Optional[str] = None,
                 text_field: str = 'text',
                 in_black_list_text: Optional[str] = None,
                 update_name: bool = True,
                 update_username: bool = True,
                 enable_update_user_data: bool = True,
                 enable_create_thread: bool = True,
                 enable_blacklist: bool = True,
                 enable_last_activity: bool = True,
                 ):
        
        self.user_model = user_model
        self.id_field = id_field
        self.username_field = username_field
        self.name_field = name_field
        self.thread_field = thread_field
        self.blacklist_field = blacklist_field
        self.last_activity_field = last_activity_field
        self.settings_model = settings_model
        self.help_chat_field = help_chat_field
        self.help_chat_id = help_chat_id
        self.text_model = text_model
        self.slug = slug
        self.text_field = text_field
        self.in_black_list_text = in_black_list_text
        self.update_name = update_name
        self.update_username = update_username
        self.enable_update_user_data = enable_update_user_data
        self.enable_create_thread = enable_create_thread
        self.enable_blacklist = enable_blacklist
        self.enable_last_activity = enable_last_activity
    async def __call__(
        self,
        handler: Callable[[TelegramObject, Dict[str, Any]], Awaitable[Any]],
        event: TelegramObject,
        data: Dict[str, Any],
        *args,
        **kwargs
    ):
        user_id = data["event_from_user"].id
        username = data["event_from_user"].username
        name = data["event_from_user"].full_name

        user, _ = await self.user_model.get_or_create(**{self.id_field: user_id})

        if self.enable_update_user_data:
            if self.update_username:
                setattr(user, self.username_field, username)
            if self.update_name:
                setattr(user, self.name_field, name)

        if self.enable_create_thread:
            thread_id = getattr(user, self.thread_field)
            if not thread_id:
                if username:
                    result = f'@{username} {user_id}'
                else:
                    result = str(user_id)
                
                if not self.help_chat_id:
                    if not self.settings_model:
                        raise ValueError("help_chat_id или settings_model должны быть предоставлены")
                    base_settings = await self.settings_model.first()
                    self.help_chat_id = getattr(base_settings, self.help_chat_field)
                
                new_thread = await event.bot.create_forum_topic(
                    chat_id=self.help_chat_id,
                    name=result,
                )
                setattr(user, self.thread_field, new_thread.message_thread_id)

        if self.enable_blacklist and getattr(user, self.blacklist_field):
            try:
                if not self.in_black_list_text:
                    if not self.text_model or not self.slug or not self.text_field:
                        raise ValueError("text_model или in_black_list_text должны быть предоставлены")
                    reply_text = await self.text_model.get(slug=self.slug)
                    reply_text = getattr(reply_text, self.text_field)
                else:
                    reply_text = self.in_black_list_text

                if isinstance(event, CallbackQuery):
                    await event.answer(
                        text=reply_text,
                        show_alert=True
                    )
                else:
                    await event.answer(
                        text=reply_text,
                    )
            except:
                pass

            return

        if self.enable_last_activity:
            setattr(user, self.last_activity_field, datetime.datetime.now(pytz.utc))

        await user.save()

        return await handler(event, data)
