import re
import datetime
from typing import Tuple, Type, Optional
from tortoise.models import Model


async def validate_phone(phone: str,
                         error_text: str = "Неверный формат номера телефона. Пожалуйста, введите номер в формате +79XXXXXXXXX или 89XXXXXXXXX",
                         ) -> Tuple[bool, str]:
    phone = re.sub(r'[\s\-]', '', phone)
    phone_pattern = re.compile(r'^(?:\+7|8)?(\d{10})$')
    
    if not phone_pattern.match(phone):
        return False, error_text
    
    if phone.startswith('8'):
        phone = '+7' + phone[1:]
    elif len(phone) == 10:
        phone = '+7' + phone
    
    return True, phone


async def validate_email(email: str,
                         error_text: str = "Неверный формат email. Пожалуйста, введите корректный email адрес",
                         ) -> Tuple[bool, str]:
    email_pattern = re.compile(r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$')
    
    if not email_pattern.match(email):
        return False, error_text
    
    return True, email.lower() 


async def extract_db_text(text_model: Type[Model], 
                          slug: str, 
                          text: Optional[str] = None,
                          text_field: str = 'text', 
                          ) -> str:
    
    curr_text = await text_model.filter(slug=slug).first()   
    if curr_text:
        return getattr(curr_text, text_field)
    else:
        if not text:
            text = 'Текст не задан'
        await text_model.create(**{'slug': slug, text_field: text})
        return text


async def extract_digits(input_string: str) -> str:
    return re.sub(r'\D', '', input_string)


async def validate_date(input_string: str,
                        error_text: str = "Неверный формат даты. Пожалуйста, введите дату в формате ДД.ММ.ГГГГ",
                        ) -> Tuple[bool, str]:
    date = await extract_digits(input_string)
    try:
        date = datetime.datetime.strptime(date, '%d%m%Y').date()
    except:
        return False, error_text

    return True, date


async def escape_markdown(text: str) -> str:
    characters_to_escape = ['_', '*', '[', ']', '`']
    for char in characters_to_escape:
        text = text.replace(char, '\\' + char)

    return text


async def escape_html(text: str) -> str:
    return re.sub(r'([&<>])', r'\\\1', text)


async def escape_markdown_v2(text: str) -> str: 
    characters_to_escape = ['_', '*', '[', ']', '`', '(', ')', '~', '>', '#', '+', '-', '=', '|', '{', '}', '.', '!', ',']
    for char in characters_to_escape:
        text = text.replace(char, '\\' + char)

    return text
